@@ -1,8 +1,8 @@
-/* NVClock 0.7 - Linux overclocker for NVIDIA cards
+/* NVClock 0.8 - Linux overclocker for NVIDIA cards
  *
  * site: http://nvclock.sourceforge.net
  *
- * Copyright(C) 2001-2003 Roderick Colenbrander
+ * Copyright(C) 2001-2005 Roderick Colenbrander
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,79 +19,44 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
+#include <stdio.h>
+#include <stdlib.h>
 #include "nvclock.h"
 #include "backend.h"
 
-typedef enum
-{
-    SET_MEMORY,
-    SET_DDR_MEMORY,
-    SET_GPU
-} Action;
 
 int CalcSpeed(int base_freq, int m, int n, int p)
 {
     return (int)((float)n/(float)m * base_freq) >> p;
 }
 
-float calc_pll(int pll, int base_freq, int device_id, short debug)
+int CalcSpeed_nv30(int base_freq, int m1, int m2, int n1, int n2, int p)
 {
+    return (int)((float)(n1*n2)/(m1*m2) * base_freq) >> p;
+}
 
-    /* NV3X */
-    if((device_id & 0xf00) == 0x300)
-    {
-	/* If bit 7 is set use the new algorithm */
-	if(nv_card.PRAMDAC[0x504/4] & 0x80)
-	{
-	    int m1, m2, n1, n2, p;
-    	    m1 = NV30_PLL_M1(pll);
-	    m2 = NV30_PLL_M2(pll);
-    	    n1 = NV30_PLL_N1(pll);
-	    n2 = NV30_PLL_N2(pll);
-    	    p = NV30_PLL_P(pll);
-	    
-	    if(debug == 1)
-		printf("m1=%d m2=%d n1=%d n2=%d p=%d\n", m1, m2, n1, n2, p);
-	    
-	    return (float)CalcSpeed_nv30(base_freq, m1, m2, n1, n2, p)/1000;
-	}
-	else
-	{
-	    int m, n, p;
-    	    m = NV30_PLL_M1(pll);
-    	    n = NV30_PLL_N1(pll);
-    	    p = NV30_PLL_P(pll);
-
-	    if(debug == 1)
-		printf("m=%d n=%d p=%d\n", m, n, p);
-	    
-	    return (float)CalcSpeed(base_freq, m, n, p)/1000;
-	}
-    }
-    else
-    {
-	int m, n, p;
-	float temp;
-	m = pll & 0xff;
-	n = (pll >> 8) & 0xff;
-	p = (pll >> 16) & 0x0f;
+/* Generic clock algorithm for all cards upto the Geforce4 */
+float GetClock(int base_freq, unsigned int pll)
+{
+    int m, n, p;
+    
+    m = pll & 0xff;
+    n = (pll >> 8) & 0xff;
+    p = (pll >> 16) & 0x0f;
 	
-	if(debug == 1)
-	    printf("m=%d n=%d p=%d\n", m, n, p);
-
-	return (float)CalcSpeed(base_freq, m, n, p)/1000;
-    }
+    if(nv_card.debug)
+        printf("m=%d n=%d p=%d\n", m, n, p);
+	
+    return (float)CalcSpeed(base_freq, m, n, p)/1000;
 }
 
 /* Calculate the requested speed. */
-static void ClockSelect(int base_freq, int clockIn, int *PLL, Action action)
+static void ClockSelect(int base_freq, int clockIn, int *PLL)
 {
     int m, n, p, bestm, bestn, bestp;
     int diff, diffOld, mlow, mhigh, nlow, nhigh, plow, phigh;
-    int level = 0;
     diffOld = clockIn;
 
-
     if(base_freq == 14318)
     {
 	mlow = 7;
@@ -176,13 +141,15 @@ static void ClockSelect(int base_freq, int clockIn, int *PLL, Action action)
 
 void set_gpu_speed(unsigned int clk)
 {
-    int PLL = nv_card.PRAMDAC[0x500/4];
+    int PLL;
 
     /* MHz -> KHz */
     clk *= 1000;
 
+    PLL = nv_card.PRAMDAC[0x500/4];
+
     /* HERE the new clocks are selected (in KHz). */
-    ClockSelect(nv_card.base_freq, clk, &PLL, SET_GPU);
+    ClockSelect(nv_card.base_freq, clk, &PLL);
 
     /* Unlock the programmable NVPLL/MPLL */
     nv_card.PRAMDAC[0x50c/4] |= 0x500;
@@ -193,7 +160,7 @@ void set_gpu_speed(unsigned int clk)
 
 void set_memory_speed(unsigned int clk)
 {
-    int PLL;
+    int PLL = nv_card.PRAMDAC[0x504/4];
 
     /* MHz -> KHz */
     clk *= 1000;
@@ -206,16 +173,11 @@ void set_memory_speed(unsigned int clk)
     if(nv_card.mem_type == SDR && ( nv_card.device_id == 0x110 || nv_card.device_id == 0x111 
 	|| nv_card.device_id == 0x172 || nv_card.device_id == 0x17a))
     {
-	if(calc_pll(nv_card.PRAMDAC[0x504/4], nv_card.base_freq, nv_card.device_id, 0) > 280000) clk *= 2;
+	if(GetClock(nv_card.base_freq, PLL) > 280) clk *= 2;
     }
 
-    PLL = nv_card.PRAMDAC[0x504/4];
-
-    /* HERE the new clocks are selected (in KHz). */
-    if(nv_card.mem_type == DDR)
-        ClockSelect(nv_card.base_freq, clk, &PLL, SET_DDR_MEMORY);
-    else
-        ClockSelect(nv_card.base_freq, clk, &PLL, SET_MEMORY);
+    /* Here the clocks are selected in kHz */
+    ClockSelect(nv_card.base_freq, clk, &PLL);
 
     /* Unlock the programmable NVPLL/MPLL */
     nv_card.PRAMDAC[0x50c/4] |= 0x500;
@@ -226,20 +188,22 @@ void set_memory_speed(unsigned int clk)
 
 float get_gpu_speed()
 {
+    int pll = nv_card.PRAMDAC[0x500/4];
+
     /* Unlock the programmable NVPLL/MPLL */
     nv_card.PRAMDAC[0x50c/4] |= 0x500;
 
     if(nv_card.debug == 1)
     {
-	printf("NVPLL_COEFF=%08x\n", nv_card.PRAMDAC[0x500/4]);
+	printf("NVPLL_COEFF=%08x\n", pll);
     }
-    
-    return ((float)calc_pll(nv_card.PRAMDAC[0x500/4], nv_card.base_freq, nv_card.device_id, nv_card.debug));
+    return (float)GetClock(nv_card.base_freq, pll);
 }
 
 float get_memory_speed()
 {
     int factor = 1;
+    int pll = nv_card.PRAMDAC[0x504/4];
 
     /* Unlock the programmable NVPLL/MPLL */
     nv_card.PRAMDAC[0x50c/4] |= 0x500;
@@ -254,7 +218,7 @@ float get_memory_speed()
     nv_card.device_id == 0x172 || nv_card.device_id == 0x17a || nv_card.device_id == 0x182 \
     || nv_card.device_id == 0x183))
     {
-	if(calc_pll(nv_card.PRAMDAC[0x504/4], nv_card.base_freq, nv_card.device_id, 0) > 280)
+	if(GetClock(nv_card.base_freq, pll) > 280)
 	{
 	    factor = 2;
 	}
@@ -265,31 +229,34 @@ float get_memory_speed()
 	printf("MPLL_COEFF=%08x\n", nv_card.PRAMDAC[0x504/4]);
     }
 
-    return ((float)calc_pll(nv_card.PRAMDAC[0x504/4], nv_card.base_freq, nv_card.device_id, nv_card.debug)) / factor;
+    return ((float)GetClock(nv_card.base_freq, pll)) / factor;
 }
 
-float get_nforce_memory_speed()
+float nforce_get_memory_speed()
 {
-    float memclk;
-    unsigned short p = (pciReadLong(0, 0, 3, 0x6c) >> 8) & 0xf;
+    unsigned short p = (pciReadLong(0x3, 0x6c) >> 8) & 0xf;
     if(!p) p = 4;
     return 400.0 / (float)p;
 }
 
-void reset_speeds()
+void reset_gpu_speed()
 {
     /* Unlock the programmable NVPLL/MPLL */
     nv_card.PRAMDAC[0x50c/4] |= 0x500;
-    
-    /* Don't support this on nv3x cards yet */
-    if((nv_card.device_id & 0xf00) == 0x300)
-	return;
-	
+
+    /* Set the gpu speed */    
     nv_card.PRAMDAC[0x500/4] = nv_card.nvpll;
+}
+
+void reset_memory_speed()
+{
+    /* Unlock the programmable NVPLL/MPLL */
+    nv_card.PRAMDAC[0x50c/4] |= 0x500;
 
     /* Don't overclock the memory of integrated GPUs */
     if(nv_card.gpu == NFORCE)
 	return;
 	
+    /* Set the memory speed */    
     nv_card.PRAMDAC[0x504/4] = nv_card.mpll;
 }
