@@ -1,11 +1,14 @@
-/*************************************************
-**      To: qt_nvclock.cpp
-** Project: nvclock
-**  Author: Jan Prokop
-**  e-mail: jprokop@ibl.sk
-*************************************************/
+/*!
+ ************************************************
+ *
+ * \file    qt_nvclock.cpp
+ * \brief   nvclock
+ * \author  Jan Prokop <jprokop@ibl.sk>
+ *
+ ************************************************/
 
-#include "qt_nvclock.moc"
+#include "qt_xfree.h"
+#include "qt_nvclock.h"
 #include "../../config.h"
 
 #include <qlabel.h>
@@ -18,26 +21,41 @@
 #include <qpushbutton.h>
 #include <qmessagebox.h>
 
+#include "x.xpm"
 #include "nv.xpm"
+#include "links.xpm"
+#include "people.xpm"
+
+//********************************************************************************
 
 static const char *txtLinks = {
-    "<qt><b>NVidia</b><br>"
+    "<qt>"
+    "<b>NVClock</b><br>"
+    "<ul>"
+    "<li> http://www.linuxhardware.org/nvclock</li>"
+    "<li> http://nvclock.sourceforge.net</li>"
+    "<li> http://projects.uid0.sk/nvclock/index.html</li>"
+    "</ul>"
+
+    "<b>NVidia</b><br>"
     "<ul>"
     "<li>home page http://www.nvidia.com</li>"
-    "<li>news http://nvclock.sourceforge.net</li>"
-    "<li>news http://news.nvidia.com</li>"
+    "<li>linux drivers http://www.nvidia.com/view.asp?IO=linux_display_archive</li>"
     "</ul>"
-    "<b>Accelerated games</b><ul>"
-    "<li>Quake2</li>"
-    "<li>Quake3 arena</li>"
-    "<li>Return To Castle Wolfrestein</li>"
+
+    "<b>Accelerated OpenGL games</b><ul>"
+    "<li>PrBoom http://prboom.sourceforge.net/about.html</li>"
+    "<li>Quake2 http://www.icculus.org/quake2</li>"
+    "<li>Quake2 http://www.idsoftware.com/games/quake/quake2</li>"
+    "<li>Quake3 Arena http://www.idsoftware.com/games/quake/quake3-arena</li>"
+    "<li>Return To Castle Wolfrestein http://www.idsoftware.com/games/wolfenstein/rtcw</li>"
+    "<li>Tux Racer http://www.tuxracer.com</li>"
     "</ul>"
     "</qt>"
 };
 
 static const char *txtAbout = {
-    "<qt>Authors<br><br>"
-    "<b>Roderick Colenbrander thunderbird2k@gmx.nospam.net</b>"
+    "<qt><b>Roderick Colenbrander thunderbird2k@gmx.nospam.net</b>"
     "<ul>"
     "<li>author of nvclock</li>"
     "<li>overclocking/pci code</li>"
@@ -61,10 +79,11 @@ static const char *txtAbout = {
 
 //********************************************************************************
 
-CTabNVidia::CTabNVidia(QWidget *parent, char *name)
+CTabNVidia::CTabNVidia(QWidget *parent, const char *name)
     : QWidget(parent, name)
 {
     l = new QGridLayout(this, 2, 3, 4, 4);
+    l->setColStretch(1, 20);
 
     int yy = 0;
     addLabel(tr("Card number"), yy);
@@ -93,6 +112,7 @@ CTabNVidia::CTabNVidia(QWidget *parent, char *name)
 
     initCardInfo(yy++);
     initAGPInfo(yy++);
+    initVideoBiosInfo(yy++);
 
     if(getCards()) slotLoad(0);
 }
@@ -101,10 +121,7 @@ CTabNVidia::CTabNVidia(QWidget *parent, char *name)
 
 void
 CTabNVidia::addLabel(const QString &text, int yy)
-{
-    QLabel *label = new QLabel(text, this);
-    l->addWidget(label, yy, 0, AlignRight | AlignVCenter);
-}
+{ l->addWidget(new QLabel(text, this), yy, 0, AlignRight | AlignVCenter); }
 
 //********************************************************************************
 
@@ -127,8 +144,9 @@ CTabNVidia::initCardInfo(int posy)
     QFont f(font());
     f.setBold(TRUE);
 
-    QLabel *label = NULL;
     ADD(labelGPUName, tr("GPU Name"));
+    ADD(labelGPUArch, tr("GPU Architecture"));
+    ADD(labelType, tr("Type"));
     ADD(labelBustype, tr("Bustype"));
     ADD(labelMemorySize, tr("Memory size"));
     ADD(labelMemoryType, tr("Memory type"));
@@ -156,7 +174,6 @@ CTabNVidia::initAGPInfo(int posy)
     QFont f(font());
     f.setBold(TRUE);
 
-    QLabel *label = NULL;
     ADD(labelAGPstatus, tr("AGP status"));
     ADD(labelSupAGPrates, tr("Supported AGP rates"));
     ADD(labelAGPrate, tr("AGP rate"));
@@ -167,33 +184,80 @@ CTabNVidia::initAGPInfo(int posy)
 
 //********************************************************************************
 
+void
+CTabNVidia::initVideoBiosInfo(int posy)
+{
+#define ADD(lname, txt) \
+    layout->addWidget(new QLabel(txt, biosInfo), yy, 0, Qt::AlignRight | Qt::AlignVCenter); \
+    lname = new QLabel(biosInfo); \
+    lname->setFont(f); \
+    layout->addWidget(lname, yy, 1); yy++;
+
+    biosInfo = new QGroupBox(tr("BIOS Info"), this);
+    l->addMultiCellWidget(biosInfo, posy, posy, 0, 2);
+
+    int yy = 0;
+    QGridLayout *layout = new QGridLayout(biosInfo, 6, 2, 4, 2);
+    layout->addRowSpacing(yy++, 10);
+
+    QFont f(font());
+    f.setBold(TRUE);
+
+    ADD(labelBiosMsg, tr("Signon message"));
+    ADD(labelBiosVersion, tr("Version"));
+    ADD(labelBiosPerf, tr("Performance level"));
+    ADD(labelBiosMask, tr("VID mask"));
+    ADD(labelBiosVLevel, tr("Voltage level"));
+    //ADD(labelBios, tr(""));
+#undef ADD
+}
+
+//********************************************************************************
+
 int
 CTabNVidia::getCards()
 {
     QString s;
-    switch(total_cards) {
-    case -1: s = tr("No nVidia card found in your system !"); break;
-    case -2: s = tr("The nvidia driver isn't loaded and the user isn't root."); break;
-    }
-    setEnabled(total_cards >= 0);
+    setEnabled(nvclock.num_cards >= 0);
 
     if(!s.isEmpty()) {
         QMessageBox::warning(this, tr("Error"), s);
         return(FALSE);
     }
 
-    for(int i=0; i <= total_cards; i++)
-        comboCardNo->insertItem(card[i].card_name);
+    for(int i=0; i < nvclock.num_cards; i++)
+        comboCardNo->insertItem(nvclock.card[i].card_name);
 
     return(TRUE);
 }
 
 //********************************************************************************
 
-int
+QString
+CTabNVidia::getType(int type) const
+{
+    QString s;
+    switch(type) {
+    case DESKTOP: s = tr("Desktop"); break;
+    case NFORCE: s = tr("NForce"); break;
+    case MOBILE: s = tr("Mobile"); break;
+    default: s = tr("Unknown");
+    }
+    return(s);
+}
+
+//********************************************************************************
+
+void
 CTabNVidia::loadCardInfo()
 {
     labelGPUName->setText(nv_card.card_name);
+
+    QString s;
+    s.sprintf("NV%X %X", nv_card.get_gpu_architecture(), nv_card.get_gpu_revision());
+    labelGPUArch->setText(s);
+
+    labelType->setText(getType(nv_card.gpu));
     labelBustype->setText(nv_card.get_bus_type());
     labelMemorySize->setText(QString("%1 MB").arg(nv_card.get_memory_size()));
     labelMemoryType->setText(QString("%1 bit %2")
@@ -207,12 +271,11 @@ CTabNVidia::loadCardInfo()
     QToolTip::add(spinCore, tr("%1 MHz - %2 MHz")
                   .arg(nv_card.nvclk_min)
                   .arg(nv_card.nvclk_max));
-    return(TRUE);
 }
 
 //********************************************************************************
 
-int
+void
 CTabNVidia::loadAGPInfo()
 {
     labelAGPstatus->setText(nv_card.get_agp_status());
@@ -225,7 +288,50 @@ CTabNVidia::loadAGPInfo()
 
     labelFWstatus->setText(QString("%1").arg(nv_card.get_fw_status()));
     labelSBAstatus->setText(QString("%1").arg(nv_card.get_sba_status()));
-    return(TRUE);
+}
+
+//********************************************************************************
+
+void
+CTabNVidia::loadVideoBIOSInfo()
+{
+    nvbios *b = nv_card.bios;
+
+    labelBiosMsg->setText(b->signon_msg);
+    labelBiosVersion->setText(b->version);
+
+    QString s, tmp;
+    for(int i=0; i<b->perf_entries; i++) {
+        performance &p = b->perf_lst[i];
+
+        if(nv_card.bios->volt_entries)
+            tmp.sprintf("%d: gpu %dMHz/memory %dMHz/%.2fV\n",
+                        i, p.nvclk, p.memclk, p.voltage);
+        else
+            tmp.sprintf("%d: %dMHz / %dMHz / %.2fV",
+                        i, p.nvclk, p.memclk, p.voltage);
+
+        s += tmp;
+    }
+    labelBiosPerf->setText(s);
+
+    //*** VID Mask ***
+    if(nv_card.bios->volt_entries)
+        s.sprintf("VID mask: %x\n", nv_card.bios->volt_mask);
+    else
+        s = QString::null;
+    labelBiosMask->setText(s);
+
+    //*** Voltage level ***
+    s = QString::null;
+    for(int i=0; i<nv_card.bios->volt_entries; i++) {
+        voltage &v = nv_card.bios->volt_lst[i];
+        tmp.sprintf("%d: %.2fV, VID: %x\n", i, v.voltage, v.VID);
+        s += tmp;
+    }
+    labelBiosVLevel->setText(s);
+
+    //labelBios->setText(s);
 }
 
 //********************************************************************************
@@ -240,7 +346,7 @@ CTabNVidia::slotLoad(int card_number)
     set_card(id);
 
     memclk = nv_card.get_memory_speed();
-    nvclk = nv_card.get_gpu_speed();
+    nvclk  = nv_card.get_gpu_speed();
 
     spinMemory->setRange(nv_card.memclk_min, nv_card.memclk_max);
     spinCore->setRange(nv_card.nvclk_min, nv_card.nvclk_max);
@@ -250,6 +356,9 @@ CTabNVidia::slotLoad(int card_number)
 
     loadCardInfo();
     loadAGPInfo();
+    
+    if(nv_card.bios)
+	loadVideoBIOSInfo();
     return(TRUE);
 }
 
@@ -259,8 +368,7 @@ int
 CTabNVidia::slotGo()
 {
     int id = comboCardNo->currentItem();
-    qDebug("overclocking card: %d", id);
-    
+
     nv_card.debug = checkDebug->isChecked();
     nv_card.set_gpu_speed(spinCore->value());
     nv_card.set_memory_speed(spinMemory->value());
@@ -268,24 +376,28 @@ CTabNVidia::slotGo()
     return(slotLoad(id));
 }
 
+//********************************************************************************
 //********************************************************************************
 
-CNVclock::CNVclock(QWidget *parent, char *name)
+CNVclock::CNVclock(QWidget *parent, const char *name)
 : QTabDialog(parent, name)
 {
-    setCaption(tr("Linux overclocker - NVclock QT "));
+    setCaption(tr("Linux Overclocker - NVclock QT "));
     
     setOkButton(0);
     setCancelButton(tr("&Quit"));
 
-    CTabNVidia *tabNVidia  = new CTabNVidia(this, "tabNVidia");
+    CTabNVidia *tabNVidia = new CTabNVidia(this);
     addTab(tabNVidia, QPixmap(nv_xpm), tr("NVidia"));
 
-    QTextBrowser *browser = new QTextBrowser(this, "tabAbout");
-    browser->setText(txtLinks);
-    addTab(browser, tr("Links"));
+    CTabXFree *tabXFree = new CTabXFree(this);
+    addTab(tabXFree, QPixmap(x_xpm), tr("XFree"));
 
-    browser = new QTextBrowser(this, "tabAbout");
+    QTextBrowser *browser = new QTextBrowser(this);
+    browser->setText(txtLinks);
+    addTab(browser, QPixmap(links_xpm), tr("Links"));
+
+    browser = new QTextBrowser(this);
     browser->setText(txtAbout);
-    addTab(browser, tr("About"));
+    addTab(browser, QPixmap(people_xpm), tr("About"));
 }
